from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import Qt
from table import ColumnsWidthSolver
from common.gui.exc import exc2signal, check_valid

class AbstractColumn:
    def __init__(self, text, visible=True, width=None, weight=None):
        self.text = text
        self.visible = visible
        self.width = width
        self.weight = weight
        self.model = None
    def set_model(self, model):
        self.model = model
    def display_data(self, row):
        return QtCore.QVariant()
    def checkstate_data(self, row):
        return QtCore.QVariant()
    def sizehint_data(self, row):
        return QtCore.QVariant()

class TableModel(QtCore.QAbstractTableModel):
    def __init__(self, columns, parent=None):
        QtCore.QAbstractTableModel.__init__(self, parent)
        self.columns = columns[:]
        for c in self.columns:
            c.set_model(self)
        self.__is_valid = True
    def isValid(self):
        return self.__is_valid
    def setValid(self, v):
        self.__is_valid = v
    def reset(self):
        self.setValid(True)
        QtCore.QAbstractTableModel.reset(self)
    @exc2signal(0)
    @check_valid(0)
    def columnCount(self, parent=QtCore.QModelIndex()):
        if parent.isValid():
            return 0
        return len(self.columns)
    @exc2signal(0)
    @check_valid(0)
    def rowCount(self, parent=QtCore.QModelIndex()):
        if parent.isValid():
            return 0
        return self.count()
    @exc2signal(QtCore.QVariant())
    @check_valid(QtCore.QVariant())
    def data(self, index, role=Qt.DisplayRole):
        row = index.row()
        column = self.columns[index.column()]
        if role == Qt.DisplayRole:
            return column.display_data(row)
        if role == Qt.CheckStateRole:
            return column.checkstate_data(row)
        if role == Qt.SizeHintRole:
            return column.sizehint_data(row)
        return QtCore.QVariant()
    @exc2signal(QtCore.QVariant())
    @check_valid(QtCore.QVariant())
    def headerData(self, section, orientation, role):
        if orientation == Qt.Vertical:
            return QtCore.QVariant()
        if role == Qt.DisplayRole:
            return QtCore.QVariant(self.columns[section].text)
        if role == Qt.FontRole:
            f = QtGui.QFont()
            f.setBold(True)
            return QtCore.QVariant(f)
        if role == ColumnsWidthSolver.WidthRole:
            return QtCore.QVariant(self.columns[section].width)
        if role == ColumnsWidthSolver.WeightRole:
            return QtCore.QVariant(self.columns[section].weight)
        if role == ColumnsWidthSolver.VisibleRole:
            return QtCore.QVariant(self.columns[section].visible)
        return QtCore.QVariant()

