"Common Ice initialization, creating proxies, etc."

#pylint: disable-msg=W0611
import Tartarus
#pylint: enable-msg=W0611
import sys, Ice

def init_communicator():
    "Initialize Ice communicator with default client configuration properties"
    idata = Ice.InitializationData()
    idata.properties = Ice.createProperties()
    idata.properties.load('/etc/Tartarus/clients/all.config')
    argv = idata.properties.parseIceCommandLineOptions(sys.argv)
    argv = idata.properties.parseCommandLineOptions('Tartarus', sys.argv)
    comm = Ice.initialize(idata)
    comm.unused_args = argv
    return comm

class Communicator:
    "Ice communicator singleton"
    __comm = None
    @classmethod
    def get(cls):
        "Return Ice communicator"
        if cls.__comm is None:
            cls.__comm = init_communicator()
        return cls.__comm
    def __init__(self):
        self.__comm = self.get()
    # pylint: disable-msg=W0622
    def make_proxy(self, property, type):
        "Make proxy by property and type. Unchecked cast is used."
        prx = self.__comm.propertyToProxy(property)
        return type.uncheckedCast(prx)
    def make_proxy_checked(self, property, type):
        "Make proxy by property and type. Checked cast is used."
        prx = self.__comm.propertyToProxy(property)
        return type.checkedCast(prx)
    # pylint: enable-msg=W0622
    def __getattr__(self, attr):
        return getattr(self.__comm, attr)

def _run_test():
    from Tartarus.iface import SysDB
    c = Communicator()
    sys.stdout.write('Create proxy without checking...')
    try:
        um = c.make_proxy('Tartarus.SysDB.UserManagerPrx', SysDB.UserManagerPrx)
    except:
        sys.stdout.write(' fail\n')
        raise
    else:
        sys.stdout.write(' done\n')
    sys.stdout.write('Ping proxy...')
    try:
        um.ice_ping()
    except:
        sys.stdout.write(' fail\n')
        raise
    else:
        sys.stdout.write(' done\n')
    sys.stdout.write('Create valid proxy with checking...')
    try:
        p = 'Tartarus.SysDB.UserManagerPrx'
        um = c.make_proxy_checked(p, SysDB.UserManagerPrx)
        um.ice_ping()
    except:
        sys.stdout.write(' fail\n')
        raise
    else:
        sys.stdout.write(' done\n')
    sys.stdout.write('Create invalid proxy with checking...')
    try:
        p = 'Tartarus.SysDB.UserManagerPrx'
        um = c.make_proxy_checked(p, SysDB.GroupManagerPrx)
    except:
        sys.stdout.write(' fail\n')
        raise
    else:
        if um is None:
            sys.stdout.write(' done\n')
        else:
            sys.stdout.write(' fail\n')


