import sys
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import Qt, SIGNAL, SLOT
from sysdb.ice import Communicator
from common.gui.ErrorDialog import ExceptionDialog
from Tartarus.iface import core, SysDB
from Ui_AddUserInfo import Ui_AddUserInfo
from Ui_AddUserPassword import Ui_AddUserPassword
from EditUserGroups import EditUserGroupsWidget
from SelectGroup import SelectGroupDialog
from sysdb.cache import GlobalGroupsCache

class AbstractPage(QtGui.QWizardPage):
    def __init__(self, parent=None):
        QtGui.QWizardPage.__init__(self, parent)
        self.setTitle('Add user wizard')

class InfoPage(AbstractPage):
    def __init__(self, parent=None):
        AbstractPage.__init__(self, parent)
        self.ui = Ui_AddUserInfo()
        self.ui.setupUi(self)
        self.setSubTitle('Common information about new user')
        self.setCommitPage(True)
        self.registerField('name*', self.ui.name)
        self.registerField('fullName', self.ui.fullName)
        self.registerField('group*', self.ui.group)
        self.connect(self.ui.checkName, SIGNAL('clicked()'), self.__checkName)
        self.connect(self.ui.name, SIGNAL('textEdited(const QString&)'), self.__nameEdited)
        self.connect(self.ui.groupSelector, SIGNAL('clicked()'), self.__selectGroup)
    def initializePage(self):
        c = Communicator()
        try:
            c.gm().getByName('users')
        except: pass
        else:
            self.ui.group.setText('users')
    def cleanupPage(self):
        self.ui.checker.setCurrentIndex(0)
    def isComplete(self):
        return self.ui.checker.currentIndex() == 1
    def __selectGroup(self):
        gid = SelectGroupDialog().get_group()
        if gid is not None:
            gname = GlobalGroupsCache().get_by_id(gid).name
            self.ui.group.setText(gname)
    def __nameEdited(self, name):
        self.ui.checker.setCurrentIndex(0)
        self.emit(SIGNAL('completeChanged()'))
    def __checkName(self):
        c = Communicator()
        name = str(self.ui.name.text())
        try:
            c.um().getByName(name)
        except core.NotFoundError:
            self.ui.checker.setCurrentIndex(1)
            self.ui.fullName.setFocus()
            self.emit(SIGNAL('completeChanged()'))
        except:
            ExceptionDialog(sys.exc_info()).exec_()
        else:
            title = 'User allready exists'
            text = 'User %s allredy exists. Choose anoser name' % name
            QtGui.QMessageBox.information(self, title, text)
            self.ui.name.setFocus()
            self.ui.name.selectAll()

class GroupsPage(AbstractPage):
    def __init__(self, parent=None):
        AbstractPage.__init__(self, parent)
        self.setSubTitle('Edit user groups')
        self.layout = QtGui.QVBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
    def initializePage(self):
        c = Communicator()
        u = SysDB.UserRecord()
        u.name = str(self.field('name').toString())
        u.fullName = str(self.field('fullName').toString())
        group = str(self.field('group').toString())
        try:
            u.gid = c.gm().getByName(group).gid
        except:
            ExceptionDialog(sys.exc_info()).exec_()
            self.emit(SIGNAL('needRestart()'))
            return
        u.shell = '/bin/bash'
        try:
            uid = c.um().create(u)
        except:
            ExceptionDialog(sys.exc_info()).exec_()
            self.emit(SIGNAL('needRestart()'))
            return
        self.wizard().user_created = True
        self.widget = EditUserGroupsWidget(uid, parent=self)
        self.layout.addWidget(self.widget)

class SecurityPage(AbstractPage):
    def __init__(self, parent=None):
        AbstractPage.__init__(self, parent)
        self.setSubTitle('Set user password')
        self.ui = Ui_AddUserPassword()
        self.ui.setupUi(self)
        self.registerField('password', self.ui.password)
        self.registerField('enabled', self.ui.enabled)
        self.connect(self.ui.password, SIGNAL('textChanged(const QString&)'), self.__textChanged)
        self.connect(self.ui.confirm, SIGNAL('textChanged(const QString&)'), self.__textChanged)
    def isComplete(self):
        password = self.ui.password.text()
        confirm = self.ui.confirm.text()
        return password == confirm and password.length() > 0
    def __textChanged(self, text):
        self.emit(SIGNAL('completeChanged()'))

class AddUserWizard(QtGui.QWizard):
    def __init__(self, parent=None, flags=Qt.WindowFlags()):
        QtGui.QWizard.__init__(self, parent, flags)
        self.__c = Communicator()
        self.addPage(InfoPage())
        gp = GroupsPage()
        self.connect(gp, SIGNAL('needRestart()'), self, SLOT('restart()'), Qt.QueuedConnection)
        self.addPage(gp)
        self.addPage(SecurityPage())
        self.user_created = False
    def exec_(self):
        ret = QtGui.QWizard.exec_(self)
        if ret == QtGui.QDialog.Accepted:
            name = str(self.field('name').toString())
            password = str(self.field('password').toString())
            enabled = self.field('enabled').toBool()
            c = Communicator()
            try:
                c.kadm().createPrincipal(name, password)
                c.kadm().setPrincEnabled(name, enabled)
            except:
                ExceptionDialog(sys.exc_info()).exec_()
        else:
            title = 'Set password warning'
            text = ('You do not set user password. '
                    'User will not be able login to system. '
                    'You can set password in the future in the user manager.')
            if self.user_created:
                QtGui.QMessageBox.information(self, title, text)
        return ret

def _run_test():
    import sys
    qa = QtGui.QApplication(sys.argv)
    qa.setOrganizationName('Etersoft')
    qa.setOrganizationDomain('tartarus.ru')
    qa.setApplicationName('testapp')
    AddUserWizard().exec_()

