from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import Qt
from sysdb.cache import GlobalUsersCache
from common.gui.TableModel import AbstractColumn, TableModel

class TextColumn(AbstractColumn):
    def __init__(self, attr_name, text, visible=True, width=None, weight=None):
        AbstractColumn.__init__(self, text, visible, width, weight)
        self.__name = attr_name
        self.__cache = GlobalUsersCache()
    def display_data(self, row):
        u = self.__cache.get(row)
        return QtCore.QVariant(getattr(u, self.__name))

class EnabledColumn(AbstractColumn):
    def __init__(self, width):
        AbstractColumn.__init__(self, 'enabled:enabled', False, width)
        self.__cache = GlobalUsersCache()
    def checkstate_data(self, row):
        u = self.__cache.get(row)
        if u.enabled:
            return QtCore.QVariant(Qt.Checked)
        return QtCore.QVariant(Qt.Unchecked)

class AllUsersModel(TableModel):
    def __init__(self, parent=None):
        self.__cache = GlobalUsersCache()
        # initialise columns
        columns = []
        columns.append(TextColumn('uid', 'uid', False, 40))
        columns.append(TextColumn('gid', 'gid', False, 40))
        columns.append(TextColumn('name', 'name', weight=1))
        columns.append(TextColumn('fullName', 'full name', weight=3))
        columns.append(TextColumn('shell', 'shell', False, weight=2))
        columns.append(EnabledColumn(30))
        TableModel.__init__(self, columns, parent)
    def count(self):
        return self.__cache.count()

def _run_test():
    import sys
    from table import UsersTableView
    qa = QtGui.QApplication(sys.argv)
    settings = QtCore.QSettings('test.org')
    table = UsersTableView()
    table.setModel(AllUsersModel(table))
    table.resize(500, 300)
    table.load(settings)
    table.show()
    qa.exec_()
    table.save(settings)

