import sys
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import SIGNAL
from Ui_GroupProperties import Ui_GroupProperties
from sysdb.cache import GlobalUsersCache, GlobalGroupsCache
from SelectUser import SelectUserDialog
from EditGroupMembers import EditGroupMembersDialog
from common.gui.ErrorDialog import ErrorDialog
from Tartarus.iface import SysDB
from sysdb.ice import Communicator

class EditGroupDialog(QtGui.QDialog):
    def __init__(self, gid, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_GroupProperties()
        self.ui.setupUi(self)
        self.okButton = self.ui.buttons.button(QtGui.QDialogButtonBox.Ok)
        self.okButton.setEnabled(False)
        self.gid = gid
        self.ucache = GlobalUsersCache()
        self.gcache = GlobalGroupsCache()
        self.__fill_props()
        self.connect(self.ui.members_button, SIGNAL('clicked()'), self.__edit_members)
        self.connect(self.ui.name_edit, SIGNAL('textEdited(const QString&)'), self.__edited)
        self.connect(self.ui.description_edit, SIGNAL('textEdited(const QString&)'), self.__edited)
    def exec_(self):
        ret = QtGui.QDialog.exec_(self)
        if ret == QtGui.QDialog.Accepted:
            self.__save()
        return ret
    def __fill_props(self):
        fromLocal8Bit = QtCore.QString.fromLocal8Bit
        g = self.gcache.get_by_id(self.gid)
        self.setWindowTitle('Edit group %s properties' % g.name)
        self.ui.name_edit.setText(fromLocal8Bit(g.name))
        self.ui.description_edit.setText(fromLocal8Bit(g.description))
    def __edited(self, text):
        self.okButton.setEnabled(True)
    def __edit_members(self):
        EditGroupMembersDialog(self.gid).exec_()
    def __save(self):
        g = SysDB.GroupRecord()
        g.gid = self.gid
        g.name = str(self.ui.name_edit.text())
        g.description = str(self.ui.description_edit.text())
        try:
            Communicator().gm().modify(g)
        except:
            exc_info = sys.exc_info()
            ErrorDialog(exc_info).exec_()

def _run_test():
    import sys
    import common.gui
    from SelectUser import SelectUserDialog
    qa = QtGui.QApplication(sys.argv)
    qa.setOrganizationName('Etersoft')
    qa.setOrganizationDomain('tartarus.ru')
    qa.setApplicationName('testapp')
    settings = QtCore.QSettings()
    uid = SelectUserDialog(settings).get_user()
    EditUserDialog(uid, settings).exec_()

