import sys
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import SIGNAL, SLOT
from Ui_GroupMembers import Ui_GroupMembers
from common.gui.TableModel import AbstractColumn, TableModel
from common.gui.ErrorDialog import ErrorDialog
from sysdb.cache import GlobalUsersCache, GlobalGroupsCache
from sysdb.ice import Communicator
from SelectUser import SelectUserDialog

class TextColumn(AbstractColumn):
    def __init__(self, name, visible=True, width=None, weight=None):
        self.__name, text = name.split(':')
        AbstractColumn.__init__(self, text, visible, width, weight)
    def display_data(self, row):
        g = self.model.users[row]
        return QtCore.QVariant(getattr(g, self.__name))

class EnabledColumn(AbstractColumn):
    def __init__(self, width):
        AbstractColumn.__init__(self, 'enabled:enabled', False, width)
        self.__cache = GlobalUsersCache()
    def checkstate_data(self, row):
        u = self.model.users[row]
        if u.enabled:
            return QtCore.QVariant(Qt.Checked)
        return QtCore.QVariant(Qt.Unchecked)

class GroupUsersModel(TableModel):
    def __init__(self, gid, parent=None):
        columns = []
        columns.append(TextColumn('uid:uid', False, 40))
        columns.append(TextColumn('gid:gid', False, 40))
        columns.append(TextColumn('name:name', weight=1))
        columns.append(TextColumn('fullName:full name', weight=3))
        columns.append(TextColumn('shelli:shell', False, weight=2))
        columns.append(EnabledColumn(30))
        TableModel.__init__(self, columns, parent)
        self.gid = gid
        self.reset()
    def reset(self):
        uids = Communicator().gm().getUsers(self.gid)
        self.users = GlobalUsersCache().get_by_ids(uids)
        TableModel.reset(self)
    def count(self):
        return len(self.users)

class EditGroupMembersWidget(QtGui.QWidget):
    def __init__(self, gid, buttons=QtGui.QDialogButtonBox.StandardButtons(), parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.gid = gid
        self.ucache = GlobalUsersCache()
        self.gcache = GlobalGroupsCache()
        self.ui = Ui_GroupMembers()
        self.ui.setupUi(self)
        name = self.gcache.get_by_id(gid).name
        self.ui.table.setModel(GroupUsersModel(gid, self))
        self.addButton = QtGui.QPushButton('Add...')
        self.addButton.setAutoDefault(False)
        self.removeButton = QtGui.QPushButton('Remove')
        self.removeButton.setAutoDefault(False)
        self.ui.buttons.setStandardButtons(buttons)
        self.ui.buttons.addButton(self.removeButton, QtGui.QDialogButtonBox.ActionRole)
        self.ui.buttons.addButton(self.addButton, QtGui.QDialogButtonBox.ActionRole)
        self.connect(self.addButton, SIGNAL('clicked()'), self.__add)
        self.connect(self.removeButton, SIGNAL('clicked()'), self.__remove)
    def load(self):
        self.ui.table.load()
    def save(self):
        self.ui.table.save()
    def __add(self):
        uid = SelectUserDialog().get_user()
        if uid is None:
            return
        c = Communicator()
        try:
            c.gm().addUsers(self.gid, [uid])
        except:
            info = sys.exc_info()
            ErrorDialog(info).exec_()
        self.ui.table.model().reset()
    def __remove(self):
        uids = self.ui.table.get_selected()
        c = Communicator()
        try:
            c.gm().delUsers(self.gid, uids)
        except:
            info = sys.exc_info()
            ErrorDialog(info).exec_()
        self.ui.table.model().reset()

class EditGroupMembersDialog(QtGui.QDialog):
    def __init__(self, gid, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.layout = QtGui.QVBoxLayout(self)
        self.layout.setSpacing(4)
        self.layout.setMargin(4)
        self.widget = EditGroupMembersWidget(gid, QtGui.QDialogButtonBox.Close)
        self.layout.addWidget(self.widget)
        name = GlobalGroupsCache().get_by_id(gid).name
        self.setWindowTitle('%s\'s members' % name)
        self.connect(self.widget.ui.buttons, SIGNAL('rejected()'), self, SLOT('reject()'))
    def exec_(self):
        settings = QtCore.QSettings()
        self.resize(settings.value('EditUserGroupsDialog/size', QtCore.QVariant(QtCore.QSize(320, 250))).toSize())
        self.widget.load()
        try:
            return QtGui.QDialog.exec_(self)
        finally:
            self.widget.save()

def _run_test():
    import sys
    import common.gui
    from SelectGroup import SelectGroupDialog
    qa = QtGui.QApplication(sys.argv)
    qa.setOrganizationName('Etersoft')
    qa.setOrganizationDomain('tartarus.ru')
    qa.setApplicationName('testapp')
    gid = SelectGroupDialog().get_group()
    if gid is not None:
        EditGroupMembersDialog(gid).exec_()

