from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import SIGNAL
from Ui_UserProperties import Ui_UserProperties
from sysdb.cache import GlobalUsersCache, GlobalGroupsCache
from SelectGroup import SelectGroupDialog
from SetPasswordDialog import SetPasswordDialog
from EditUserGroups import EditUserGroupsDialog
from Tartarus.iface import SysDB
from sysdb.ice import Communicator
from common.gui.ErrorDialog import ErrorDialog
import sys

class EditUserDialog(QtGui.QDialog):
    def __init__(self, uid, settings=None, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_UserProperties()
        self.ui.setupUi(self)
        self.okButton = self.ui.buttons.button(QtGui.QDialogButtonBox.Ok)
        self.okButton.setEnabled(False)
        self.uid = uid
        self.ucache = GlobalUsersCache()
        self.gcache = GlobalGroupsCache()
        self.__fill_props()
        self.__settings = settings
        self.connect(self.ui.groupToolButton, SIGNAL('clicked()'), self.__select_group)
        self.connect(self.ui.passwordButton, SIGNAL('clicked()'), self.__set_password)
        self.connect(self.ui.groupsButton, SIGNAL('clicked()'), self.__edit_groups)
        self.connect(self.ui.fullNameEdit, SIGNAL('textEdited(const QString&)'), lambda text: self.__edited())
        self.connect(self.ui.shellEdit, SIGNAL('textEdited(const QString&)'), lambda text: self.__edited())
    def exec_(self):
        ret = QtGui.QDialog.exec_(self)
        if ret == QtGui.QDialog.Accepted:
            self.__save()
        return ret
    def __select_group(self):
        gid = SelectGroupDialog(self.__settings, self).get_group()
        if gid is not None:
            self.ui.groupEdit.setText(str(self.gcache.get_by_id(gid).name))
            self.__edited()
    def __fill_props(self):
        fromUtf8 = QtCore.QString.fromUtf8
        u = self.ucache.get_by_id(self.uid)
        self.setWindowTitle('Edit user %s properties' % u.name)
        self.ui.nameEdit.setText(fromUtf8(str(u.name)))
        self.ui.fullNameEdit.setText(fromUtf8(str(u.fullName)))
        self.ui.groupEdit.setText(fromUtf8(str(self.gcache.get_by_id(u.gid).name)))
        self.ui.shellEdit.setText(fromUtf8(str(u.shell)))
    def __edited(self):
        self.okButton.setEnabled(True)
    def __set_password(self):
        SetPasswordDialog(self.uid, self).exec_()
    def __edit_groups(self):
        EditUserGroupsDialog(self.uid).exec_()
    def __save(self):
        u = SysDB.UserRecord()
        u.uid = self.uid
        gname = str(self.ui.groupEdit.text())
        u.gid = self.gcache.get_by_name(gname).gid
        u.name = str(self.ui.nameEdit.text())
        u.fullName = str(self.ui.fullNameEdit.text())
        u.shell = str(self.ui.shellEdit.text())
        try:
            Communicator().um().modify(u)
        except:
            exc_info = sys.exc_info()
            ErrorDialog(exc_info).exec_()

def _run_test():
    import sys
    import common.gui
    from SelectUser import SelectUserDialog
    qa = QtGui.QApplication(sys.argv)
    qa.setOrganizationName('Etersoft')
    qa.setOrganizationDomain('tartarus.ru')
    qa.setApplicationName('testapp')
    settings = QtCore.QSettings()
    uid = SelectUserDialog(settings).get_user()
    EditUserDialog(uid, settings).exec_()

