import sys
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import SIGNAL, SLOT
from Ui_UserGroups import Ui_UserGroups
from common.gui.TableModel import AbstractColumn, TableModel
from common.gui.ErrorDialog import ErrorDialog
from sysdb.cache import GlobalUsersCache, GlobalGroupsCache
from sysdb.ice import Communicator
from SelectGroup import SelectGroupDialog

class TextColumn(AbstractColumn):
    def __init__(self, name, visible=True, width=None, weight=None):
        self.__name, text = name.split(':')
        AbstractColumn.__init__(self, text, visible, width, weight)
    def display_data(self, row):
        g = self.model.groups[row]
        return QtCore.QVariant(getattr(g, self.__name))

class UserGroupsModel(TableModel):
    def __init__(self, uid, parent=None):
        columns = []
        columns.append(TextColumn('gid:gid', visible=False, width=40))
        columns.append(TextColumn('name:name', weight=1))
        columns.append(TextColumn('description:description', weight=3))
        TableModel.__init__(self, columns, parent)
        self.uid = uid
        self.reset()
    def reset(self):
        gids = Communicator().gm().getGroupsForUserId(self.uid)
        self.groups = GlobalGroupsCache().get_by_ids(gids)
        TableModel.reset(self)
    def count(self):
        return len(self.groups)

class EditUserGroupsWidget(QtGui.QWidget):
    def __init__(self, uid, buttons=QtGui.QDialogButtonBox.StandardButtons(), parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.uid = uid
        self.ucache = GlobalUsersCache()
        self.gcache = GlobalGroupsCache()
        self.ui = Ui_UserGroups()
        self.ui.setupUi(self)
        name = self.ucache.get_by_id(uid).name
        self.ui.table.setModel(UserGroupsModel(uid, self))
        self.addButton = QtGui.QPushButton('Add...')
        self.addButton.setAutoDefault(False)
        self.removeButton = QtGui.QPushButton('Remove')
        self.removeButton.setAutoDefault(False)
        self.ui.buttons.setStandardButtons(buttons)
        self.ui.buttons.addButton(self.removeButton, QtGui.QDialogButtonBox.ActionRole)
        self.ui.buttons.addButton(self.addButton, QtGui.QDialogButtonBox.ActionRole)
        self.connect(self.addButton, SIGNAL('clicked()'), self.__add)
        self.connect(self.removeButton, SIGNAL('clicked()'), self.__remove)
    def load(self):
        self.ui.table.load()
    def save(self):
        self.ui.table.save()
    def __add(self):
        gid = SelectGroupDialog(QtCore.QSettings()).get_group()
        c = Communicator()
        try:
            c.gm().addUsers(gid, [self.uid])
        except:
            info = sys.exc_info()
            ErrorDialog(info).exec_()
        self.ui.table.model().reset()
    def __remove(self):
        gids = self.ui.table.get_selected()
        c = Communicator()
        try:
            c.gm().delUserFromGroups(self.uid, gids)
        except:
            info = sys.exc_info()
            ErrorDialog(info).exec_()
        self.ui.table.model().reset()

class EditUserGroupsDialog(QtGui.QDialog):
    def __init__(self, uid, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.layout = QtGui.QVBoxLayout(self)
        self.layout.setSpacing(4)
        self.layout.setMargin(4)
        self.widget = EditUserGroupsWidget(uid, QtGui.QDialogButtonBox.Close)
        self.layout.addWidget(self.widget)
        name = GlobalUsersCache().get_by_id(uid).name
        self.setWindowTitle('%s\'s groups' % name)
        self.connect(self.widget.ui.buttons, SIGNAL('rejected()'), self, SLOT('reject()'))
    def exec_(self):
        settings = QtCore.QSettings()
        self.resize(settings.value('EditUserGroupsDialog/size', QtCore.QVariant(QtCore.QSize(320, 250))).toSize())
        self.widget.load()
        try:
            return QtGui.QDialog.exec_(self)
        finally:
            self.widget.save()

def _run_test():
    import sys
    import common.gui
    from SelectUser import SelectUserDialog
    qa = QtGui.QApplication(sys.argv)
    qa.setOrganizationName('Etersoft')
    qa.setOrganizationDomain('tartarus.ru')
    qa.setApplicationName('testapp')
    uid = SelectUserDialog().get_user()
    if uid is not None:
        EditUserGroupsDialog(uid).exec_()

