from __future__ import with_statement
from contextlib import nested
from PyQt4 import QtCore
from common.gui.TableModel import AbstractColumn, TableModel
from sysdb.cache import GlobalGroupsCache
from AllGroupsModel import AllGroupsModel
import common.gui.settings as s

class TextColumn(AbstractColumn):
    def __init__(self, name, items, visible=True, width=None, weight=None):
        self.__name, text = name.split(':')
        AbstractColumn.__init__(self, text, visible, width, weight)
        self.__items = items
    def display_data(self, row):
        u = self.__items[row]
        return QtCore.QVariant(getattr(u, self.__name))

class StaticGroupsModel(TableModel):
    def __init__(self, parent=None):
        self.items = []
        columns = []
        columns.append(TextColumn('gid:gid', self.items, visible=False, width=40))
        columns.append(TextColumn('name:name', self.items, weight=1))
        columns.append(TextColumn('description:description', self.items, weight=3))
        TableModel.__init__(self, columns, parent)
    def count(self):
        return len(self.items)
    
class RecentGroupsModel(StaticGroupsModel):
    def __init__(self, settings=None, parent=None):
        StaticGroupsModel.__init__(self, parent)
        self.cache = GlobalGroupsCache()
    def save(self):
        settings = QtCore.QSettings()
        group = 'Groups/Recent'
        with nested(s.group(settings, group),
                s.write_array(settings, 'items', len(self.items))):
            for i in xrange(len(self.items)):
                settings.setArrayIndex(i)
                settings.setValue('gid', QtCore.QVariant(self.items[i].gid))
    def load(self):
        settings = QtCore.QSettings()
        group = 'Groups/Recent'
        with nested(s.group(settings, group),
                s.read_array(settings, 'items')) as (foo, size):
            del self.items[:]
            for i in xrange(size):
                settings.setArrayIndex(i)
                gid, ok = settings.value('gid', QtCore.QVariant('-1')).toInt()
                try:
                    g = self.cache.get_by_id(gid)
                except: pass
                else:
                    self.items.append(g)
    def add(self, gid):
        found = False
        for g in self.items:
            if g.gid == gid:
                self.items.remove(g)
                self.items.insert(0, g)
                found = True
                break
        if not found:
            g = self.cache.get_by_id(gid)
            self.items.insert(0, g)
            if len(self.items) > 10:
                del self.items[10:]

class SearchGroupsModel(StaticGroupsModel):
    def __init__(self, parent=None):
        StaticGroupsModel.__init__(self, parent)
        self.cache = GlobalGroupsCache()
    def search(self, search_str):
        self.items[:] = self.cache.search(search_str)
        self.reset()

