from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import Qt, SIGNAL
import SelectUser
import SelectGroup
from AddUser import AddUserWizard
from EditUser import EditUserDialog
from EditGroup import EditGroupDialog
from AddGroup import AddGroupDialog
from common.gui.ErrorDialog import ExceptionDialog
from sysdb.ice import Communicator
from sysdb.cache import GlobalUsersCache, GlobalGroupsCache

def action(text):
    def wrapper(method):
        attrname = '__action_' + method.__name__
        def m(inst):
            if not hasattr(inst, attrname):
                a = QtGui.QAction(text, MainWindow.get())
                setattr(inst, attrname, a)
                QtCore.QObject.connect(a, SIGNAL('triggered()'), method.__get__(inst, type(inst)))
            return getattr(inst, attrname)
        return m
    return wrapper

class ActionManager:
    __instance = None
    @staticmethod
    def get():
        return ActionManager.__instance
    def __init__(self):
        if ActionManager.__instance is None:
            ActionManager.__instance = self
        else:
            raise RuntimeError('Atempt to create more then one ActionManager instance')
    @action('Add user...')
    def addUser(self):
        AddUserWizard().exec_()
        MainWindow.get().reset()
    @action('Edit user...')
    def editUser(self):
        uid = MainWindow.get().ui.widgets.widget(0).get_selected()[0]
        MainWindow.get().ui.widgets.widget(0).addToRecent(uid)
        EditUserDialog(uid).exec_()
        MainWindow.get().reset()
    @action('Remove user...')
    def removeUser(self):
        uid = MainWindow.get().ui.widgets.widget(0).get_selected()[0]
        name = GlobalUsersCache().get_by_id(uid).name
        ret = QtGui.QMessageBox.question(MainWindow.get(),
                'Warning', 'Do you really want to remove %s user' % name,
                QtGui.QMessageBox.Yes|QtGui.QMessageBox.No)
        if ret == QtGui.QMessageBox.Yes:
            c = Communicator()
            c.kadm().deletePrincipal(name)
            c.um().delete(uid)
        MainWindow.get().reset()
    @action('Add group...')
    def addGroup(self):
        AddGroupDialog().exec_()
        MainWindow.get().reset()
    @action('Edit group...')
    def editGroup(self):
        gid = MainWindow.get().ui.widgets.widget(1).get_selected()[0]
        MainWindow.get().ui.widgets.widget(1).addToRecent(gid)
        EditGroupDialog(gid).exec_()
        MainWindow.get().reset()
    @action('Remove group...')
    def removeGroup(self):
        gid = MainWindow.get().ui.widgets.widget(1).get_selected()[0]
        name = GlobalGroupsCache().get_by_id(gid).name
        ret = QtGui.QMessageBox.question(MainWindow.get(),
                'Warning', 'Do you really want to remove %s group' % name,
                QtGui.QMessageBox.Yes|QtGui.QMessageBox.No)
        if ret == QtGui.QMessageBox.Yes:
            c = Communicator()
            c.gm().delete(gid)
        MainWindow.get().reset()
    @action('Update')
    def update(self):
        MainWindow.get().reset()

class SelectUserWidget(SelectUser.SelectUserWidget):
    def __init__(self, parent=None):
        SelectUser.SelectUserWidget.__init__(self, parent)
        self.menu = QtGui.QMenu()
        actions = ActionManager.get()
        self.menu.addAction(actions.addUser())
        self.menu.addAction(actions.editUser())
        self.menu.addAction(actions.removeUser())
        self.connect(self.ui.table, SIGNAL('customContextMenuRequested(const QPoint&)'), self.__context_menu)
        self.connect(self.ui.table, SIGNAL('doubleClicked(const QModelIndex&)'), self.__doubleclicked)
    def __context_menu(self, point):
        point = self.ui.table.viewport().mapToGlobal(point)
        self.menu.popup(point)
    def __doubleclicked(self, index):
        uid = self.get_selected()[0]
        self.addToRecent(uid)
        EditUserDialog(uid).exec_()

class SelectGroupWidget(SelectGroup.SelectGroupWidget):
    def __init__(self, parent=None):
        SelectGroup.SelectGroupWidget.__init__(self, parent)
        self.menu = QtGui.QMenu()
        actions = ActionManager.get()
        self.menu.addAction(actions.addGroup())
        self.menu.addAction(actions.editGroup())
        self.menu.addAction(actions.removeGroup())
        self.connect(self.ui.table, SIGNAL('customContextMenuRequested(const QPoint&)'), self.__context_menu)
        self.connect(self.ui.table, SIGNAL('doubleClicked(const QModelIndex&)'), self.__doubleclicked)
    def __context_menu(self, point):
        point = self.ui.table.viewport().mapToGlobal(point)
        self.menu.popup(point)
    def __doubleclicked(self, index):
        gid = self.get_selected()[0]
        self.addToRecent(gid)
        EditGroupDialog(gid).exec_()

from Ui_MainWindow import Ui_MainWindow

class MainWindow(QtGui.QMainWindow):
    __instance = None
    @staticmethod
    def get():
        return MainWindow.__instance
    def __init__(self, actions, parent=None, flags=Qt.WindowFlags()):
        QtGui.QMainWindow.__init__(self, parent, flags)
        if MainWindow.__instance is None:
            MainWindow.__instance = self
        else:
            raise RuntimeError('Atempt to create more then one MainWindow instance')
        self.actions = actions
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.ui.toolbar.addAction(actions.addUser())
        self.ui.toolbar.addAction(actions.editUser())
        self.ui.toolbar.addAction(actions.removeUser())
        self.ui.toolbar.addSeparator()
        self.ui.toolbar.addAction(actions.addGroup())
        self.ui.toolbar.addAction(actions.editGroup())
        self.ui.toolbar.addAction(actions.removeGroup())
        self.ui.toolbar.addSeparator()
        self.ui.toolbar.addAction(actions.update())
        self.ui.actions.addAction(actions.addUser())
        self.ui.actions.addAction(actions.editUser())
        self.ui.actions.addAction(actions.removeUser())
        self.ui.actions.addSeparator()
        self.ui.actions.addAction(actions.addGroup())
        self.ui.actions.addAction(actions.editGroup())
        self.ui.actions.addAction(actions.removeGroup())
        self.ui.actions.addSeparator()
        self.ui.actions.addAction(actions.update())
        actions.editUser().setEnabled(False)
        actions.removeUser().setEnabled(False)
        actions.editGroup().setEnabled(False)
        actions.removeGroup().setEnabled(False)
        for i in xrange(2):
            self.ui.widgets.widget(i).load()
        self.connect(self.ui.selectUserWidget, SIGNAL('Exception'), self.__exception)
        self.connect(self.ui.tree, SIGNAL('currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)'), self.__tree_item_changed)
        self.connect(self.ui.widgets.widget(0), SIGNAL('selectedUsersChanged'), self.__selectedUsersChanged)
        self.connect(self.ui.widgets.widget(1), SIGNAL('selectedGroupsChanged'), self.__selectedGroupsChanged)
    def reset(self):
        self.ui.widgets.currentWidget().reset()
        GlobalUsersCache().reset()
        GlobalGroupsCache().reset()
    def __exception(self, exc_info):
        ExceptionDialog(exc_info).exec_()
    def __tree_item_changed(self, current, previous):
        if current.text(0) == 'Users':
            self.ui.widgets.setCurrentIndex(0)
            self.ui.widgets.widget(1).reset()
        else:
            self.ui.widgets.setCurrentIndex(1)
            self.ui.widgets.widget(0).reset()
    def __del__(self):
        for i in xrange(2):
            self.ui.widgets.widget(i).save()
    def __selectedUsersChanged(self, uids):
        if len(uids) > 0:
            for i in (self.actions.editUser(), self.actions.removeUser()):
                i.setEnabled(True)
        else:
            for i in (self.actions.editUser(), self.actions.removeUser()):
                i.setEnabled(False)
    def __selectedGroupsChanged(self, gids):
        if len(gids) > 0:
            for i in (self.actions.editGroup(), self.actions.removeGroup()):
                i.setEnabled(True)
        else:
            for i in (self.actions.editGroup(), self.actions.removeGroup()):
                i.setEnabled(False)

def _run_test():
    import sys
    qa = QtGui.QApplication(sys.argv)
    qa.setOrganizationName('Etersoft')
    qa.setOrganizationDomain('tartarus.ru')
    qa.setApplicationName('testapp')
    actions = ActionManager()
    w = MainWindow(actions)
    w.show()
    qa.exec_()

