from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import Qt
from PyQt4.QtCore import Qt, SIGNAL, SLOT
from Ui_SelectGroup import Ui_SelectGroup
from GroupsModels import *
from sysdb.cache import GlobalGroupsCache

class SelectGroupWidget(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.ui = Ui_SelectGroup()
        self.ui.setupUi(self)
        self.cache = GlobalGroupsCache()
        self.models = []
        self.models = [RecentGroupsModel(), AllGroupsModel()]
        self.ui.table.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.ui.table.setModel(self.models[1])
        self.connect(self.ui.table.selectionModel(), SIGNAL('selectionChanged(const QItemSelection&,const QItemSelection&)'), self.__selectionChanged)
        self.connect(self.ui.pageSelector, SIGNAL('currentIndexChanged(int)'), self.__currentIndexChanged)
        self.connect(self.ui.searchButton, SIGNAL('clicked()'), self.__search)
        self.connect(self.ui.searchEdit, SIGNAL('returnPressed()'), self.__search)
    def get_selected(self):
        sm = self.ui.table.selectionModel()
        srows = sm.selectedRows(0)
        if len(srows) > 0:
            return [self.ui.table.model().data(i).toInt()[0] for i in srows]
        else:
            return []
    def addToRecent(self, gid):
        self.models[0].add(gid)
    def load(self):
        self.models[0].load()
        self.ui.table.load()
        if self.models[0].rowCount() > 0:
            self.ui.pageSelector.setCurrentIndex(0)
    def save(self):
        self.models[0].save()
        self.ui.table.save()
    def reset(self):
        self.models[1].reset()
    def __currentIndexChanged(self, index):
        table = self.ui.table
        table.save()
        table.setModel(self.models[index])
        self.connect(table.selectionModel(), SIGNAL('selectionChanged(const QItemSelection&,const QItemSelection&)'), self.__selectionChanged)
        self.__selectionChanged()
        table.load()
        if self.ui.pageSelector.count()-1 != self.ui.pageSelector.currentIndex():
            self.__del_search_result()
    def __search(self):
        sstr = str(self.ui.searchEdit.text())
        self.__add_search_result()
        self.models[-1].search(sstr)
        self.ui.pageSelector.setCurrentIndex(len(self.models)-1)
        self.__selectionChanged()
        if self.models[-1].rowCount() == 1:
            t = self.ui.table
            sm = t.selectionModel()
            t.selectionModel().select(t.model().index(0, 0), sm.Select|sm.Rows)
            t.setFocus(Qt.ShortcutFocusReason)
        elif self.models[-1].rowCount() > 1:
            pass
        elif self.models[-1].rowCount() == 0:
            self.ui.searchEdit.selectAll()
    def __has_search_result(self):
        return isinstance(self.models[-1], SearchGroupsModel)
    def __add_search_result(self):
        if self.__has_search_result():
            return
        self.models.append(SearchGroupsModel())
        self.ui.pageSelector.addItem('Search result')
    def __del_search_result(self):
        if not self.__has_search_result():
            return
        del self.models[-1]
        self.ui.pageSelector.removeItem(len(self.models))
    def __selectionChanged(self, *args):
        self.emit(SIGNAL('selectedGroupsChanged'), self.get_selected())


class SelectGroupDialog(QtGui.QDialog):
    def __init__(self, settings=None, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.settings = settings
        self.layout = QtGui.QVBoxLayout(self)
        self.layout.setMargin(4)
        self.widget = SelectGroupWidget(self)
        self.layout.addWidget(self.widget)
        BB = QtGui.QDialogButtonBox
        self.buttons = QtGui.QDialogButtonBox(BB.Ok|BB.Cancel, Qt.Horizontal, self)
        self.layout.addWidget(self.buttons)
        self.okButton = self.buttons.button(BB.Ok)
        self.okButton.setEnabled(False)
        self.resize(250, 300)
        self.connect(self.widget, SIGNAL('selectedGroupsChanged'), self.__selection_changed)
        self.connect(self.buttons, SIGNAL('accepted()'), self, SLOT('accept()'))
        self.connect(self.buttons, SIGNAL('rejected()'), self, SLOT('reject()'))
        self.connect(self.widget.ui.table, SIGNAL('doubleClicked(const QModelIndex&)'), self.__doubleclicked)
    def get_group(self):
        self.widget.load()
        gid = None
        if self.exec_() == QtGui.QDialog.Accepted:
            gid = self.widget.get_selected()[0]
            self.widget.addToRecent(gid)
        self.widget.save()
        return gid
    def __doubleclicked(self, index):
        self.accept()
    def __selection_changed(self, gids):
        self.okButton.setEnabled(len(gids)>0)

def _run_test():
    import sys
    qa = QtGui.QApplication(sys.argv)
    qa.setOrganizationName('Etersoft')
    qa.setOrganizationDomain('tartarus.ru')
    qa.setApplicationName('testapp')
    settings = QtCore.QSettings()
    w = SelectGroupDialog(settings)
    print w.get_group()
    settings.sync()

