from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import Qt, SIGNAL, SLOT
from Ui_SelectUser import Ui_SelectUser
from UsersModels import *
from sysdb.cache import GlobalUsersCache
from common.gui.ErrorDialog import ErrorDialog

class SelectUserWidget(QtGui.QWidget):
    def __init__(self, settings=None, parent=None):
        QtGui.QWidget.__init__(self, parent)
        if settings is not None:
            self.settings = settings
        else:
            self.settings = QtCore.QSettings()
        self.ui = Ui_SelectUser()
        self.ui.setupUi(self)
        self.cache = GlobalUsersCache()
        self.models = []
        self.models = [RecentUsersModel(), AllUsersModel()]
        self.ui.table.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.connect(self.ui.table, SIGNAL('Exception'), self.__table_exception, Qt.QueuedConnection)
        self.connect(self.ui.table, SIGNAL('selectionChanged'), self.__selectionChanged)
        self.connect(self.ui.pageSelector, SIGNAL('currentIndexChanged(int)'), self.__currentIndexChanged)
        self.connect(self.ui.searchButton, SIGNAL('clicked()'), self.__search)
        self.connect(self.ui.searchEdit, SIGNAL('returnPressed()'), self.__search)
    def showEvent(self, event):
        if not event.spontaneous():
            if self.models[0].rowCount() > 0:
                self.ui.table.setModel(self.models[0])
                self.ui.pageSelector.setCurrentIndex(0)
            else:
                self.ui.table.setModel(self.models[1])
                self.ui.pageSelector.setCurrentIndex(1)
        return QtGui.QWidget.showEvent(self, event)
    def get_selected(self):
        sm = self.ui.table.selectionModel()
        srows = sm.selectedRows(0)
        if len(srows) > 0:
            return [self.ui.table.model().data(i).toInt()[0] for i in srows]
        else:
            return []
    def addToRecent(self, uid):
        self.models[0].add(uid)
    def load(self):
        self.models[0].load(self.settings)
    def save(self):
        self.models[0].save(self.settings)
        self.ui.table.save()
    def reset(self):
        self.models[1].reset()
        self.__selectionChanged()
    def __currentIndexChanged(self, index):
        table = self.ui.table
        table.setModel(self.models[index])
        self.connect(table.selectionModel(), SIGNAL('selectionChanged(const QItemSelection&,const QItemSelection&)'), self.__selectionChanged)
        self.__selectionChanged()
        if self.ui.pageSelector.count()-1 != self.ui.pageSelector.currentIndex():
            self.__del_search_result()
    def __search(self):
        sstr = str(self.ui.searchEdit.text())
        self.__add_search_result()
        self.models[-1].search(sstr)
        self.ui.pageSelector.setCurrentIndex(len(self.models)-1)
        self.__selectionChanged()
        if self.models[-1].rowCount() == 1:
            t = self.ui.table
            sm = t.selectionModel()
            t.selectionModel().select(t.model().index(0, 0), sm.Select|sm.Rows)
            t.setFocus(Qt.ShortcutFocusReason)
        elif self.models[-1].rowCount() > 1:
            pass
        elif self.models[-1].rowCount() == 0:
            self.ui.searchEdit.selectAll()
    def __has_search_result(self):
        return isinstance(self.models[-1], SearchUsersModel)
    def __add_search_result(self):
        if self.__has_search_result():
            return
        self.models.append(SearchUsersModel())
        self.ui.pageSelector.addItem('Search result')
    def __del_search_result(self):
        if not self.__has_search_result():
            return
        del self.models[-1]
        self.ui.pageSelector.removeItem(len(self.models))
    def __selectionChanged(self, *args):
        self.emit(SIGNAL('selectedUsersChanged'), self.get_selected())
    def __table_exception(self, exc_info):
        self.emit(SIGNAL('Exception'), exc_info)

class SelectUserDialog(QtGui.QDialog):
    def __init__(self, settings=None, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.settings = settings
        self.layout = QtGui.QVBoxLayout(self)
        self.layout.setMargin(4)
        self.widget = SelectUserWidget(settings, self)
        self.layout.addWidget(self.widget)
        BB = QtGui.QDialogButtonBox
        self.buttons = QtGui.QDialogButtonBox(BB.Ok|BB.Cancel, Qt.Horizontal, self)
        self.layout.addWidget(self.buttons)
        self.okButton = self.buttons.button(BB.Ok)
        self.okButton.setEnabled(False)
        self.resize(250, 300)
        self.connect(self.widget, SIGNAL('Exception'), self.__table_exception)
        self.connect(self.widget, SIGNAL('selectedUsersChanged'), self.__selection_changed)
        self.connect(self.buttons, SIGNAL('accepted()'), self, SLOT('accept()'))
        self.connect(self.buttons, SIGNAL('rejected()'), self, SLOT('reject()'))
        self.connect(self.widget.ui.table, SIGNAL('doubleClicked(const QModelIndex&)'), self.__double_clicked)
    def get_user(self):
        self.widget.load()
        uid = None
        if self.exec_() == QtGui.QDialog.Accepted:
            uid = self.widget.get_selected()[0]
            self.widget.addToRecent(uid)
        self.widget.save()
        return uid
    def __double_clicked(self, index):
        self.accept()
    def __selection_changed(self, uids):
        self.okButton.setEnabled(len(uids)>0)
    def __table_exception(self, exc_info):
        ErrorDialog(exc_info).exec_()
        if self.isVisible():
            self.reject()

def _run_test():
    import sys
    qa = QtGui.QApplication(sys.argv)
    qa.setOrganizationName('Etersoft')
    qa.setOrganizationDomain('tartarus.ru')
    qa.setApplicationName('testapp')
    settings = QtCore.QSettings()
    w = SelectUserDialog(settings)
    print w.get_user()
    settings.sync()

