from Ui_SetPassword import Ui_SetPassword
from PyQt4 import QtGui
from PyQt4.QtCore import SIGNAL
from sysdb.ice import Communicator
from sysdb.cache import GlobalUsersCache
from Tartarus.iface import core

class SetPasswordDialog(QtGui.QDialog):
    def __init__(self, uid, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_SetPassword()
        self.ui.setupUi(self)
        self.uid = uid
        self.name = GlobalUsersCache().get_by_id(self.uid).name
        self.ui.nameEdit.setText(self.name)
        self.okButton = self.ui.buttons.button(QtGui.QDialogButtonBox.Ok)
        self.okButton.setEnabled(False)
        self.connect(self.ui.passwordEdit, SIGNAL('textEdited(const QString&)'), self.__pwd_edited)
        self.connect(self.ui.confirmEdit, SIGNAL('textEdited(const QString&)'), self.__pwd_edited)
    def exec_(self):
        ret = QtGui.QDialog.exec_(self)
        if ret == QtGui.QDialog.Accepted:
            pwd = self.ui.passwordEdit.text()
            self.__set_password(str(pwd))
        return ret
    def __set_password(self, password):
        c = Communicator()
        try:
            c.kadm().createPrincipal(self.name, password)
        except core.DBError:
            # XXX: change ^^^ to AlreadyExistsError after server testing
            c.kadm().changePrincPassword(self.name, password)
    def __pwd_edited(self, text):
        pwd = self.ui.passwordEdit.text()
        cfm = self.ui.confirmEdit.text()
        e = (pwd.length() > 0 and pwd == cfm)
        self.okButton.setEnabled(e)

def _run_test():
    import sys
    from PyQt4 import QtCore
    from SelectUser import SelectUserDialog
    qa = QtGui.QApplication(sys.argv)
    qa.setOrganizationName('Etersoft')
    qa.setOrganizationDomain('tartarus.ru')
    qa.setApplicationName('testapp')
    settings = QtCore.QSettings()
    uid = SelectUserDialog(settings).get_user()
    if uid is not None:
        SetPasswordDialog(uid).exec_()

