from __future__ import with_statement
from contextlib import nested
from PyQt4 import QtCore
from common.gui.TableModel import AbstractColumn, TableModel
from sysdb.cache import GlobalUsersCache
from AllUsersModel import AllUsersModel
import common.gui.settings as s

class TextColumn(AbstractColumn):
    def __init__(self, name, items, visible=True, width=None, weight=None):
        self.__name, text = name.split(':')
        AbstractColumn.__init__(self, text, visible, width, weight)
        self.__items = items
    def display_data(self, row):
        u = self.__items[row]
        return QtCore.QVariant(getattr(u, self.__name))

class EnabledColumn(AbstractColumn):
    def __init__(self, items, width):
        AbstractColumn.__init__(self, 'enabled', False, width)
        self.__items = items
    def checkstate_data(self, row):
        u = self.__items[row]
        if u.enabled:
            return QtCore.QVariant(Qt.Checked)
        return QtCore.QVariant(Qt.Unchecked)

class StaticUsersModel(TableModel):
    def __init__(self, parent=None):
        self.items = []
        columns = []
        columns.append(TextColumn('uid:uid', self.items, visible=False, width=40))
        columns.append(TextColumn('gid:gid', self.items, visible=False, width=40))
        columns.append(TextColumn('name:name', self.items, weight=1))
        columns.append(TextColumn('fullName:full name', self.items, weight=3))
        columns.append(TextColumn('shell:shell', self.items, visible=False, weight=2))
        columns.append(EnabledColumn(self.items, 30))
        TableModel.__init__(self, columns, parent)
    def count(self):
        return len(self.items)
    
class RecentUsersModel(StaticUsersModel):
    def __init__(self, parent=None):
        StaticUsersModel.__init__(self, parent)
        self.cache = GlobalUsersCache()
    def save(self, settings, group='Users/Recent'):
        with nested(s.group(settings, group),
                s.write_array(settings, 'items', len(self.items))):
            for i in xrange(len(self.items)):
                settings.setArrayIndex(i)
                settings.setValue('uid', QtCore.QVariant(self.items[i].uid))
    def load(self, settings, group='Users/Recent'):
        with nested(s.group(settings, group),
                s.read_array(settings, 'items')) as (foo, size):
            del self.items[:]
            for i in xrange(size):
                settings.setArrayIndex(i)
                uid, ok = settings.value('uid', QtCore.QVariant('-1')).toInt()
                try:
                    u = self.cache.get_by_id(uid)
                except: pass
                else:
                    self.items.append(u)
    def add(self, uid):
        found = False
        for u in self.items:
            if u.uid == uid:
                self.items.remove(u)
                self.items.insert(0, u)
                found = True
                break
        if not found:
            u = self.cache.get_by_id(uid)
            self.items.insert(0, u)
            if len(self.items) > 10:
                del self.items[10:]

class SearchUsersModel(StaticUsersModel):
    def __init__(self, parent=None):
        StaticUsersModel.__init__(self, parent)
        self.cache = GlobalUsersCache()
    def search(self, search_str):
        self.items[:] = self.cache.search(search_str)
        self.reset()

